#!/usr/bin/env python

#********************************************************************
# Filename: 	   	sqlmigratelite/migrate.py
# Authors: 			Daniel Sisco
# Date Created:   	9/19/2010
# 
# Abstract: Provides a base class for Elixir-based sqlite database migration
#
# Copyright 2010 Daniel Sisco
# This file is part of elixirmigrate.
#
# elixirmigrate is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# elixirmigrate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with elixirmigrate.  If not, see <http://www.gnu.org/licenses/>.
#********************************************************************
from sqlalchemy import create_engine
from sqlalchemy import Table, Column, Integer, String, MetaData, ForeignKey
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, relationship, backref
import shutil
import tempfile

Base = declarative_base()

#********************************************************************
class MigrateObject():
	"""
	Uninstantiated base class for the sqlmigratelite module. Provides common
	methods for migration of sqlite datbases
	"""
	
	def __init__(self, dbPath, dbVersion):
		self.localDbPath    = dbPath
		self.localDbVersion = dbVersion
		self.localDict      = {}
		
	def dump(self):
		"""
		Dumps database content into a dictionary object with single-key/single-value pairs. 
		Key values are database table names and values are lists of table rows (contents).
		"""
		
		# dump preamble
		shutil.copy(self.localDbPath, self.localDbPath+"."+self.localDbVersion)
		
		# SQLAlchemy connection
		self.connString = 'sqlite:///' + self.localDbPath
		self.engine = create_engine(self.connString, echo=False)
		self.SessionObject = sessionmaker(bind=self.engine)
		self.session = self.SessionObject()
		
		# dump custom content
		self.dumpContent()
		
		# dump epilogue
		self.session.close()
		return self.localDict
	
	def load(self, dict):
		"""
		Loads database content from a dictionary object into a database of the specified name.
		"""
		
		# load preamble
		self.localDict = dict
		self.tempDbFile = tempfile.mkstemp(".temp", "", ".")
		self.connString = 'sqlite:///' + self.tempDbFile[1]

		self.engine = create_engine(self.connString, echo=False)
		self.SessionObject = sessionmaker(bind=self.engine)
		self.session = self.SessionObject()

		# load custom conent
		self.loadContent()
		
		# load epilogue
		self.session.close()
		shutil.copy(self.tempDbFile[1], self.localDbPath)
